package kr.co.vividnext.sodalive.audition.applicant

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import coil.load
import coil.transform.CircleCropTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.databinding.ItemAuditionApplicantBinding

class AuditionApplicantListAdapter(
    private var itemList: List<GetAuditionRoleApplicantItem>,
    private val onClickVote: (Int) -> Unit,
    private val onClickPlayOrPause: (Int) -> Unit
) : RecyclerView.Adapter<AuditionApplicantListAdapter.ViewHolder>() {

    private var currentPlayingIndex: Int = -1

    inner class DiffCallback(
        private val oldList: List<GetAuditionRoleApplicantItem>,
        private val newList: List<GetAuditionRoleApplicantItem>
    ) : DiffUtil.Callback() {
        override fun getOldListSize() = oldList.size
        override fun getNewListSize() = newList.size

        override fun areItemsTheSame(
            oldItemPosition: Int,
            newItemPosition: Int
        ) = oldList[oldItemPosition].applicantId == newList[newItemPosition].applicantId

        override fun areContentsTheSame(
            oldItemPosition: Int,
            newItemPosition: Int
        ) = oldList[oldItemPosition] == newList[newItemPosition]
    }

    inner class ViewHolder(
        private val binding: ItemAuditionApplicantBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(item: GetAuditionRoleApplicantItem, position: Int) {
            binding.ivVote.setOnClickListener { onClickVote(position) }
            binding.ivPlayOrPause.setOnClickListener { onClickPlayOrPause(position) }

            binding.tvNickname.text = item.nickname
            binding.ivProfile.load(item.profileImageUrl) {
                crossfade(true)
                placeholder(R.drawable.bg_placeholder)
                transformations(CircleCropTransformation())
            }

            binding.ivPlayOrPause.setImageResource(
                if (position == currentPlayingIndex) {
                    R.drawable.ic_audition_play
                } else {
                    R.drawable.ic_audition_pause
                }
            )
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int) = ViewHolder(
        ItemAuditionApplicantBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        )
    )

    override fun getItemCount() = itemList.size

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.bind(itemList[position], position)
    }

    @SuppressLint("NotifyDataSetChanged")
    fun updateData(newData: List<GetAuditionRoleApplicantItem>) {
        itemList = newData
        notifyDataSetChanged()
    }

    fun updatePlayingIndex(newIndex: Int) {
        val previousIndex = currentPlayingIndex
        currentPlayingIndex = newIndex
        notifyItemChanged(previousIndex)
        notifyItemChanged(currentPlayingIndex)
    }
}
