package kr.co.vividnext.sodalive.audition.role

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.Toast
import coil.load
import coil.transform.RoundedCornersTransformation
import kr.co.vividnext.sodalive.R
import kr.co.vividnext.sodalive.audition.applicant.AuditionApplicantListAdapter
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivityAuditionRoleDetailBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject

class AuditionRoleDetailActivity : BaseActivity<ActivityAuditionRoleDetailBinding>(
    ActivityAuditionRoleDetailBinding::inflate
) {
    private val viewModel: AuditionRoleDetailViewModel by inject()

    private lateinit var loadingDialog: LoadingDialog
    private lateinit var adapter: AuditionApplicantListAdapter

    private var auditionRoleId: Long = 0
    private var isOpenInformation = false

    override fun onCreate(savedInstanceState: Bundle?) {
        auditionRoleId = intent.getLongExtra(Constants.EXTRA_AUDITION_ROLE_ID, 0)

        if (auditionRoleId <= 0) {
            Toast.makeText(
                applicationContext,
                "잘못된 요청입니다.\n다시 시도해 주세요.",
                Toast.LENGTH_LONG
            ).show()

            finish()
        }

        super.onCreate(savedInstanceState)

        bindData()
        viewModel.getAuditionRoleDetail(auditionRoleId = auditionRoleId)
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.toolbar.tvBack.setOnClickListener { finish() }
        binding.tvOpen.setOnClickListener {
            isOpenInformation = !isOpenInformation
            if (isOpenInformation) {
                binding.tvInformation.maxLines = Int.MAX_VALUE
                binding.tvOpen.text = "접기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_top,
                    0,
                    0,
                    0
                )
            } else {
                binding.tvInformation.maxLines = 3
                binding.tvOpen.text = "펼치기"
                binding.tvOpen.setCompoundDrawablesWithIntrinsicBounds(
                    R.drawable.ic_live_detail_bottom,
                    0,
                    0,
                    0
                )
            }
        }


    }

    private fun bindData() {
        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.auditionRoleDetailLiveData.observe(this) { roleDetail ->
            binding.toolbar.tvBack.text = roleDetail.name

            binding.tvAuditionScript.setOnClickListener {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(roleDetail.auditionScriptUrl)))
            }

            binding.tvOriginalWork.setOnClickListener {
                startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(roleDetail.originalWorkUrl)))
            }

            binding.tvInformation.text = roleDetail.information

            binding.ivCover.load(roleDetail.imageUrl) {
                crossfade(true)
                placeholder(R.drawable.bg_placeholder)
                transformations(RoundedCornersTransformation(5f.dpToPx()))
            }

            if (roleDetail.isAlreadyApplicant) {
                binding.tvApplicant.text = "오디션 재지원"
            } else {
                binding.tvApplicant.text = "오디션 지원하기"
            }
        }

        viewModel.totalCountLiveData.observe(this) {
            binding.groupApplicant.visibility = View.GONE
            binding.groupNoApplicant.visibility = View.GONE

            if (it > 0) {
                binding.groupApplicant.visibility = View.VISIBLE
                binding.tvApplicantCount.text = "$it"
            } else {
                binding.groupNoApplicant.visibility = View.VISIBLE
            }
        }

        viewModel.applicantListLiveData.observe(this) {
        }
    }
}
