package kr.co.vividnext.sodalive.audition.role

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.gson.annotations.SerializedName
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audition.AuditionRepository
import kr.co.vividnext.sodalive.audition.applicant.GetAuditionRoleApplicantItem
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class AuditionRoleDetailViewModel(private val repository: AuditionRepository) : BaseViewModel() {
    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private val _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private val _auditionRoleDetailLiveData = MutableLiveData<GetAuditionRoleDetailResponse>()
    val auditionRoleDetailLiveData: LiveData<GetAuditionRoleDetailResponse>
        get() = _auditionRoleDetailLiveData

    private val _totalCountLiveData = MutableLiveData(0)
    val totalCountLiveData: LiveData<Int>
        get() = _totalCountLiveData

    private val _applicantListLiveData = MutableLiveData<List<GetAuditionRoleApplicantItem>>()
    val applicantListLiveData: LiveData<List<GetAuditionRoleApplicantItem>>
        get() = _applicantListLiveData

    private val _sortTypeLiveData = MutableLiveData(AuditionApplicantSortType.NEWEST)
    val sortTypeLiveData: LiveData<AuditionApplicantSortType>
        get() = _sortTypeLiveData

    var page = 1
    var isLast = false
    private var auditionRoleId = -1L
    private val pageSize = 10

    fun getAuditionRoleDetail(auditionRoleId: Long, onFailure: (() -> Unit)? = null) {
        this.auditionRoleId = auditionRoleId
        _isLoading.value = true

        val auditionRoleDetail = repository.getAuditionRoleDetail(
            auditionRoleId = auditionRoleId,
            token = "Bearer ${SharedPreferenceManager.token}"
        )

        val getApplicantList = repository.getAuditionApplicantList(
            auditionRoleId = auditionRoleId,
            sortType = _sortTypeLiveData.value!!,
            page = page,
            size = pageSize,
            token = "Bearer ${SharedPreferenceManager.token}"
        )

        compositeDisposable.add(
            Observable.combineLatest(
                auditionRoleDetail.toObservable(),
                getApplicantList.toObservable()
            ) { response1, response2 ->
                Pair(response1, response2)
            }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    { pair ->
                        val roleDetailResponse = pair.first
                        val applicantListResponse = pair.second

                        if (roleDetailResponse.success && roleDetailResponse.data != null) {
                            _auditionRoleDetailLiveData.value = roleDetailResponse.data!!
                        } else {
                            if (roleDetailResponse.message != null) {
                                _toastLiveData.value = roleDetailResponse.message
                            } else {
                                _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            }

                            if (onFailure != null) {
                                onFailure()
                            }
                        }

                        if (applicantListResponse.success && applicantListResponse.data != null) {
                            val data = applicantListResponse.data

                            _totalCountLiveData.value = data.totalCount
                            _applicantListLiveData.value = data.items
                            page += 1
                        } else {
                            if (applicantListResponse.message != null) {
                                _toastLiveData.value = applicantListResponse.message
                            } else {
                                _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            }

                            if (onFailure != null) {
                                onFailure()
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                        if (onFailure != null) {
                            onFailure()
                        }
                    }
                )
        )
    }

    fun getAuditionApplicantList() {
        _isLoading.value = true

        compositeDisposable.add(
            repository.getAuditionApplicantList(
                auditionRoleId = auditionRoleId,
                sortType = _sortTypeLiveData.value!!,
                page = page,
                size = pageSize,
                token = "Bearer ${SharedPreferenceManager.token}"
            )
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(
                    {
                        if (it.success && it.data != null) {
                            val data = it.data

                            _totalCountLiveData.value = data.totalCount
                            _applicantListLiveData.value = data.items
                            page += 1
                        } else {
                            if (it.message != null) {
                                _toastLiveData.value = it.message
                            } else {
                                _toastLiveData.value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                            }
                        }

                        _isLoading.value = false
                    },
                    {
                        _isLoading.value = false
                        it.message?.let { message -> Logger.e(message) }
                        _toastLiveData.postValue("알 수 없는 오류가 발생했습니다. 다시 시도해 주세요.")
                    }
                )
        )
    }

    fun setSortType(sortType: AuditionApplicantSortType) {
        val prevSortType = _sortTypeLiveData.value!!

        if (sortType != prevSortType) {
            _sortTypeLiveData.value = sortType
            isLast = false
            page = 1
            getAuditionApplicantList()
        }
    }

    enum class AuditionApplicantSortType {
        @SerializedName("NEWEST")
        NEWEST,

        @SerializedName("LIKES")
        LIKES
    }
}
